@@ -10,6 +10,9 @@ use LWP::UserAgent;
 use URI;
 use XML::LibXML ;
 use Geo::Coder::GoogleMaps::Location;
+use Geo::Coder::GoogleMaps::Response;
+
+=encoding utf-8
 
 =head1 NAME
 
@@ -17,14 +20,20 @@ Geo::Coder::GoogleMaps - Google Maps Geocoding API
 
 =head1 VERSION
 
-Version 0.12
+Version 0.3
 
 =cut
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 
 =head1 SYNOPSIS
 
+WARNING WARNING WARNING
+
+	There is a huge API change between version 0.2 and 0.3 ! Please see the documentation of the geocode() method !
+
+WARNING WARNING WARNING
+
 This module provide Google Maps API. Please note that this module use Tatsuhiko Miyagawa's work on Geo::Coder::Google as base (L<http://search.cpan.org/~miyagawa/>).
 
 In fact it's a fork of Mr Miyagawa's module. Geo::Coder::GoogleMaps use the default JSON data type as default output but also support XML/KML.
@@ -35,12 +44,12 @@ The direct output of the geocode() method is now a Geo::Coder::GoogleMaps::Locat
 	use Geo::Coder::GoogleMaps;
 	
 	my $gmap = Geo::Coder::GoogleMaps->new( apikey => 'abcd' , output => 'xml');
-	my $location = $gmap->geocode(location => '88 rue du chateau, 92600, Asnières sur seine, France');
-	
-	print $location->latitude,',',$location->longitude,"\n";
-	
-	$location->toKML()->toString(); # is absolutly equivalent to $location->toKML(1);
-	
+	my $response = $gmap->geocode(location => '88 rue du chateau, 92600, Asnières sur seine, France');
+	if( $response->is_success() ){
+		my $location = $response->placemarks()->[0];
+		print $location->latitude,',',$location->longitude,"\n";
+		$location->toKML()->toString(); # is absolutly equivalent to $location->toKML(1);
+	}
 
 =head1 FUNCTIONS
 
@@ -66,7 +75,7 @@ sub new {
 	my $key = delete $param{apikey}
 		or Carp::croak("Usage: new(apikey => \$apikey)");
 	
-	my $ua   = delete $param{ua}   || LWP::UserAgent->new(agent => __PACKAGE__ . "/$VERSION");
+	my $ua   = delete $param{ua}   || LWP::UserAgent->new(agent => "Mozilla/5.0 (compatible;Geo::Coder::GoogleMaps/$Geo::Coder::GoogleMaps::VERSION");
 	my $host = delete $param{host} || 'maps.google.com';
 	my $output = delete $param{output} || 'json';
 	
@@ -75,7 +84,13 @@ sub new {
 
 =head2 geocode
 
-Get a location from the Google Maps API. It return a list (in list context) of Geo::Code::GoogleMaps::Location objects or the first one in a scalar context.
+WARNING WARNING WARNING
+
+	There is a huge API change between version 0.2 and 0.3 ! This method do not returns placemarks directly anymore !!
+
+WARNING WARNING WARNING
+
+Get a location from the Google Maps API. It return a Geo::Code::GoogleMaps::Response object.
 
 	my $location = $gmap->geocode(location => '88 rue du chateau, 92600, Asnières sur seine, France');
 	print $location->Serialyze ;
@@ -106,7 +121,7 @@ sub geocode {
 	}
 	
 	my $uri = URI->new("http://$self->{host}/maps/geo");
-	$uri->query_form(q => $location, key => $self->{key});
+	$uri->query_form(q => $location, key => $self->{key},sensor => "false", output => "json");
 # 	$uri->query_form(q => $location, output => $self->{output}, key => $self->{key});
 	
 	my $res = $self->{ua}->get($uri);
@@ -124,13 +139,20 @@ sub geocode {
 	local $JSON::Syck::ImplicitUnicode = 1;
 	my $data = JSON::Syck::Load($content);
 	
+# 	print "[Debug] JSON::Syck::Load()-ed data:\n",Data::Dumper::Dumper( $data ),"\n";
+	
+	my $response = Geo::Coder::GoogleMaps::Response->new(status_code => $data->{Status}->{code}, status_request => $data->{Status}->{request} );
 	my @placemark=();
 	foreach my $Placemark (@{$data->{Placemark}}){
 		my $loc = Geo::Coder::GoogleMaps::Location->new(output => $self->{output});
 		$loc->_setData($Placemark);
+# 		print "[Debug] JSON::Syck::Load()-ed data:\n",Data::Dumper::Dumper( $Placemark ),"\n";
+		$response->add_placemark($loc);
 		push @placemark, $loc;
 	}
-	wantarray ? @placemark : $placemark[0];
+# 	print "[debug] new response object:\n",Data::Dumper::Dumper($response),"\n";
+	return $response;
+# 	wantarray ? @placemark : $placemark[0];
 }
 
 1;
@@ -140,7 +162,7 @@ __END__
 
 L<Geo::Coder::Google> (the original module) is from Tatsuhiko Miyagawa.
 
-Arnaud DUPUIS, C<< <a.dupuis at nabladev.com> >>
+Arnaud Dupuis, C<< <a.dupuis at infinityperl.org> >>
 
 =head1 BUGS
 
@@ -160,9 +182,17 @@ You can also look for information at:
 
 =over 4
 
-=item * Nabla Development: 
+=item * Infinity Perl: 
 
-L<http://www.nabladev.com> and L<http://opensource.nabladev.com>
+L<http://www.infinityperl.org>
+
+=item * Google Code repository
+
+L<https://code.google.com/p/geo-coder-googlemaps/>
+
+=item * Google Maps API documentation
+
+L<http://code.google.com/apis/maps/documentation/geocoding/>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -186,7 +216,7 @@ L<http://search.cpan.org/dist/Geo-Coder-GoogleMaps>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2007 Arnaud DUPUIS and Nabla Development, all rights reserved.
+Copyright 2007-2010 Arnaud DUPUIS, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
